@@ -93,7 +93,7 @@ class CommandSMembers : public Commander {
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    *output = redis::MultiBulkString(members, false);
+    *output = conn->SetOfBulkStrings(members);
     return Status::OK();
   }
 };
@@ -171,12 +171,12 @@ class CommandSPop : public Commander {
     }
 
     if (with_count_) {
-      *output = redis::MultiBulkString(members, false);
+      *output = conn->SetOfBulkStrings(members);
     } else {
       if (members.size() > 0) {
         *output = redis::BulkString(members.front());
       } else {
-        *output = redis::NilString();
+        *output = conn->NilString();
       }
     }
     return Status::OK();
@@ -211,7 +211,7 @@ class CommandSRandMember : public Commander {
     if (!s.ok()) {
       return {Status::RedisExecErr, s.ToString()};
     }
-    *output = redis::MultiBulkString(members, false);
+    *output = conn->SetOfBulkStrings(members);
     return Status::OK();
   }
 
@@ -249,7 +249,7 @@ class CommandSDiff : public Commander {
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    *output = redis::MultiBulkString(members, false);
+    *output = conn->SetOfBulkStrings(members);
     return Status::OK();
   }
 };
@@ -269,7 +269,7 @@ class CommandSUnion : public Commander {
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    *output = redis::MultiBulkString(members, false);
+    *output = conn->SetOfBulkStrings(members);
     return Status::OK();
   }
 };
@@ -289,7 +289,7 @@ class CommandSInter : public Commander {
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    *output = redis::MultiBulkString(members, false);
+    *output = conn->SetOfBulkStrings(members);
     return Status::OK();
   }
 };
@@ -432,13 +432,13 @@ class CommandSScan : public CommandSubkeyScanBase {
       return {Status::RedisExecErr, s.ToString()};
     }
 
-    *output = CommandScanBase::GenerateOutput(srv, members, CursorType::kTypeSet);
+    *output = CommandScanBase::GenerateOutput(srv, conn, members, CursorType::kTypeSet);
     return Status::OK();
   }
 };
 
 REDIS_REGISTER_COMMANDS(MakeCmdAttr<CommandSAdd>("sadd", -3, "write", 1, 1, 1),
-                        MakeCmdAttr<CommandSRem>("srem", -3, "write", 1, 1, 1),
+                        MakeCmdAttr<CommandSRem>("srem", -3, "write no-dbsize-check", 1, 1, 1),
                         MakeCmdAttr<CommandSCard>("scard", 2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandSMembers>("smembers", 2, "read-only", 1, 1, 1),
                         MakeCmdAttr<CommandSIsMember>("sismember", 3, "read-only", 1, 1, 1),
