@@ -64,7 +64,8 @@ class CommandScanBase : public Commander {
     }
   }
 
-  std::string GenerateOutput(Server *srv, const std::vector<std::string> &keys, CursorType cursor_type) const {
+  std::string GenerateOutput(Server *srv, const Connection *conn, const std::vector<std::string> &keys,
+                             CursorType cursor_type) const {
     std::vector<std::string> list;
     if (keys.size() == static_cast<size_t>(limit_)) {
       auto end_cursor = srv->GenerateCursorFromKeyName(keys.back(), cursor_type);
@@ -73,7 +74,7 @@ class CommandScanBase : public Commander {
       list.emplace_back(redis::BulkString("0"));
     }
 
-    list.emplace_back(redis::MultiBulkString(keys, false));
+    list.emplace_back(ArrayOfBulkStrings(keys));
 
     return redis::Array(list);
   }
@@ -111,25 +112,11 @@ class CommandSubkeyScanBase : public CommandScanBase {
     return Commander::Parse(args);
   }
 
-  std::string GenerateOutput(Server *srv, const std::vector<std::string> &fields,
-                             const std::vector<std::string> &values, CursorType cursor_type) {
-    std::vector<std::string> list;
-    auto items_count = fields.size();
-    if (items_count == static_cast<size_t>(limit_)) {
-      auto end_cursor = srv->GenerateCursorFromKeyName(fields.back(), cursor_type);
-      list.emplace_back(redis::BulkString(end_cursor));
-    } else {
-      list.emplace_back(redis::BulkString("0"));
+  std::string GetNextCursor(Server *srv, std::vector<std::string> &fields, CursorType cursor_type) const {
+    if (fields.size() == static_cast<size_t>(limit_)) {
+      return srv->GenerateCursorFromKeyName(fields.back(), cursor_type);
     }
-    std::vector<std::string> fvs;
-    if (items_count > 0) {
-      for (size_t i = 0; i < items_count; i++) {
-        fvs.emplace_back(fields[i]);
-        fvs.emplace_back(values[i]);
-      }
-    }
-    list.emplace_back(redis::MultiBulkString(fvs, false));
-    return redis::Array(list);
+    return "0";
   }
 
  protected:
