@@ -62,6 +62,13 @@ StatusOr<std::unique_ptr<T>> WrapOutPtrToUnique(Args&&... args) {
   return rocksdb::DB::OpenForReadOnly(db_options, dbname, column_families, handles, dbptr);
 }
 
+[[nodiscard]] inline rocksdb::Status DBOpenForSecondaryInstance(
+    const rocksdb::DBOptions& db_options, const std::string& dbname, const std::string& secondary_path,
+    const std::vector<rocksdb::ColumnFamilyDescriptor>& column_families,
+    std::vector<rocksdb::ColumnFamilyHandle*>* handles, rocksdb::DB** dbptr) {
+  return rocksdb::DB::OpenAsSecondary(db_options, dbname, secondary_path, column_families, handles, dbptr);
+}
+
 }  // namespace details
 
 inline StatusOr<std::unique_ptr<rocksdb::DB>> DBOpen(const rocksdb::Options& options, const std::string& dbname) {
@@ -95,6 +102,19 @@ inline StatusOr<std::unique_ptr<rocksdb::DB>> DBOpenForReadOnly(
       Status::DBOpenErr>(db_options, dbname, column_families, handles);
 }
 
+inline StatusOr<std::unique_ptr<rocksdb::DB>> DBOpenAsSecondaryInstance(
+    const rocksdb::DBOptions& db_options, const std::string& dbname, const std::string& secondary_path,
+    const std::vector<rocksdb::ColumnFamilyDescriptor>& column_families,
+    std::vector<rocksdb::ColumnFamilyHandle*>* handles) {
+  return details::WrapOutPtrToUnique<
+      rocksdb::DB,
+      static_cast<rocksdb::Status (*)(const rocksdb::DBOptions&, const std::string&, const std::string&,
+                                      const std::vector<rocksdb::ColumnFamilyDescriptor>&,
+                                      std::vector<rocksdb::ColumnFamilyHandle*>*, rocksdb::DB**)>(
+          details::DBOpenForSecondaryInstance),
+      Status::DBOpenErr>(db_options, dbname, secondary_path, column_families, handles);
+}
+
 inline StatusOr<std::unique_ptr<rocksdb::BackupEngine>> BackupEngineOpen(rocksdb::Env* db_env,
                                                                          const rocksdb::BackupEngineOptions& options) {
   return details::WrapOutPtrToUnique<
