@@ -41,15 +41,18 @@ enum BitOpFlags {
 
 namespace redis {
 
+// We use least-significant bit (LSB) numbering (also known as bit-endianness).
+// This means that within a group of 8 bits, we read right-to-left.
+// This is different from applying "bit" commands to string, which uses MSB.
 class Bitmap : public Database {
  public:
   class SegmentCacheStore;
 
   Bitmap(engine::Storage *storage, const std::string &ns) : Database(storage, ns) {}
-  rocksdb::Status GetBit(const Slice &user_key, uint32_t offset, bool *bit);
+  rocksdb::Status GetBit(const Slice &user_key, uint32_t bit_offset, bool *bit);
   rocksdb::Status GetString(const Slice &user_key, uint32_t max_btos_size, std::string *value);
-  rocksdb::Status SetBit(const Slice &user_key, uint32_t offset, bool new_bit, bool *old_bit);
-  rocksdb::Status BitCount(const Slice &user_key, int64_t start, int64_t stop, uint32_t *cnt);
+  rocksdb::Status SetBit(const Slice &user_key, uint32_t bit_offset, bool new_bit, bool *old_bit);
+  rocksdb::Status BitCount(const Slice &user_key, int64_t start, int64_t stop, bool is_bit_index, uint32_t *cnt);
   rocksdb::Status BitPos(const Slice &user_key, bool bit, int64_t start, int64_t stop, bool stop_given, int64_t *pos);
   rocksdb::Status BitOp(BitOpFlags op_flag, const std::string &op_name, const Slice &user_key,
                         const std::vector<Slice> &op_keys, int64_t *len);
@@ -63,7 +66,7 @@ class Bitmap : public Database {
                                    std::vector<std::optional<BitfieldValue>> *rets) {
     return bitfield<true>(user_key, ops, rets);
   }
-  static bool GetBitFromValueAndOffset(const std::string &value, uint32_t offset);
+  static bool GetBitFromValueAndOffset(std::string_view value, uint32_t bit_offset);
   static bool IsEmptySegment(const Slice &segment);
 
  private:
