@@ -24,6 +24,7 @@
 
 #include <cstdint>
 
+#include "common/bit_util.h"
 #include "redis_string.h"
 #include "server/redis_reply.h"
 #include "storage/redis_metadata.h"
@@ -31,33 +32,28 @@
 
 namespace redis {
 
-rocksdb::Status BitmapString::GetBit(const std::string &raw_value, uint32_t offset, bool *bit) {
-  auto string_value = raw_value.substr(Metadata::GetOffsetAfterExpire(raw_value[0]));
-  uint32_t byte_index = offset >> 3;
-  uint32_t bit_val = 0;
-  uint32_t bit_offset = 7 - (offset & 0x7);
+rocksdb::Status BitmapString::GetBit(const std::string &raw_value, uint32_t bit_offset, bool *bit) {
+  std::string_view string_value = std::string_view{raw_value}.substr(Metadata::GetOffsetAfterExpire(raw_value[0]));
+  uint32_t byte_index = bit_offset >> 3;
   if (byte_index < string_value.size()) {
-    bit_val = string_value[byte_index] & (1 << bit_offset);
+    *bit = util::msb::GetBit(reinterpret_cast<const uint8_t *>(string_value.data()), bit_offset);
+  } else {
+    *bit = false;
   }
-  *bit = bit_val != 0;
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status BitmapString::SetBit(const Slice &ns_key, std::string *raw_value, uint32_t offset, bool new_bit,
+rocksdb::Status BitmapString::SetBit(const Slice &ns_key, std::string *raw_value, uint32_t bit_offset, bool new_bit,
                                      bool *old_bit) {
   size_t header_offset = Metadata::GetOffsetAfterExpire((*raw_value)[0]);
   auto string_value = raw_value->substr(header_offset);
-  uint32_t byte_index = offset >> 3;
+  uint32_t byte_index = bit_offset >> 3;
   if (byte_index >= string_value.size()) {  // expand the bitmap
     string_value.append(byte_index - string_value.size() + 1, 0);
   }
-  uint32_t bit_offset = 7 - (offset & 0x7);
-  auto byteval = string_value[byte_index];
-  *old_bit = (byteval & (1 << bit_offset)) != 0;
-
-  byteval = static_cast<char>(byteval & (~(1 << bit_offset)));
-  byteval = static_cast<char>(byteval | ((new_bit & 0x1) << bit_offset));
-  string_value[byte_index] = byteval;
+  auto *data_ptr = reinterpret_cast<uint8_t *>(string_value.data());
+  *old_bit = util::msb::GetBit(data_ptr, bit_offset);
+  util::msb::SetBitTo(data_ptr, bit_offset, new_bit);
 
   *raw_value = raw_value->substr(0, header_offset);
   raw_value->append(string_value);
@@ -68,45 +64,53 @@ rocksdb::Status BitmapString::SetBit(const Slice &ns_key, std::string *raw_value
   return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status BitmapString::BitCount(const std::string &raw_value, int64_t start, int64_t stop, uint32_t *cnt) {
+rocksdb::Status BitmapString::BitCount(const std::string &raw_value, int64_t start, int64_t stop, bool is_bit_index,
+                                       uint32_t *cnt) {
   *cnt = 0;
-  auto string_value = raw_value.substr(Metadata::GetOffsetAfterExpire(raw_value[0]));
-  /* Convert negative indexes */
-  if (start < 0 && stop < 0 && start > stop) {
-    return rocksdb::Status::OK();
-  }
+  std::string_view string_value = std::string_view{raw_value}.substr(Metadata::GetOffsetAfterExpire(raw_value[0]));
   auto strlen = static_cast<int64_t>(string_value.size());
-  if (start < 0) start = strlen + start;
-  if (stop < 0) stop = strlen + stop;
-  if (start < 0) start = 0;
-  if (stop < 0) stop = 0;
-  if (stop >= strlen) stop = strlen - 1;
+  int64_t totlen = strlen;
+  if (is_bit_index) totlen <<= 3;
+  std::tie(start, stop) = NormalizeRange(start, stop, totlen);
+  // Always return 0 if start is greater than stop after normalization.
+  if (start > stop) return rocksdb::Status::OK();
+
+  /* By default:
+   * start means start byte in bitmap, stop means stop byte in bitmap.
+   * When is_bit_index is true, start and stop means start bit and stop bit.
+   * So it should be normalized bit range to byte range. */
+  int64_t start_byte = start;
+  int64_t stop_byte = stop;
+  uint8_t first_byte_neg_mask = 0, last_byte_neg_mask = 0;
+  std::tie(start_byte, stop_byte) =
+      NormalizeToByteRangeWithPaddingMask(is_bit_index, start, stop, &first_byte_neg_mask, &last_byte_neg_mask);
 
   /* Precondition: end >= 0 && end < strlen, so the only condition where
    * zero can be returned is: start > stop. */
-  if (start <= stop) {
-    int64_t bytes = stop - start + 1;
-    *cnt = RawPopcount(reinterpret_cast<const uint8_t *>(string_value.data()) + start, bytes);
+  int64_t bytes = stop_byte - start_byte + 1;
+  *cnt = util::RawPopcount(reinterpret_cast<const uint8_t *>(string_value.data()) + start_byte, bytes);
+  if (first_byte_neg_mask != 0 || last_byte_neg_mask != 0) {
+    uint8_t firstlast[2] = {0, 0};
+    if (first_byte_neg_mask != 0) firstlast[0] = string_value[start_byte] & first_byte_neg_mask;
+    if (last_byte_neg_mask != 0) firstlast[1] = string_value[stop_byte] & last_byte_neg_mask;
+    *cnt -= util::RawPopcount(firstlast, 2);
   }
+
   return rocksdb::Status::OK();
 }
 
 rocksdb::Status BitmapString::BitPos(const std::string &raw_value, bool bit, int64_t start, int64_t stop,
                                      bool stop_given, int64_t *pos) {
-  auto string_value = raw_value.substr(Metadata::GetOffsetAfterExpire(raw_value[0]));
+  std::string_view string_value = std::string_view{raw_value}.substr(Metadata::GetOffsetAfterExpire(raw_value[0]));
   auto strlen = static_cast<int64_t>(string_value.size());
-  /* Convert negative indexes */
-  if (start < 0) start = strlen + start;
-  if (stop < 0) stop = strlen + stop;
-  if (start < 0) start = 0;
-  if (stop < 0) stop = 0;
-  if (stop >= strlen) stop = strlen - 1;
+  /* Convert negative and out-of-bound indexes */
+  std::tie(start, stop) = NormalizeRange(start, stop, strlen);
 
   if (start > stop) {
     *pos = -1;
   } else {
     int64_t bytes = stop - start + 1;
-    *pos = RawBitpos(reinterpret_cast<const uint8_t *>(string_value.data()) + start, bytes, bit);
+    *pos = util::msb::RawBitpos(reinterpret_cast<const uint8_t *>(string_value.data()) + start, bytes, bit);
 
     /* If we are looking for clear bits, and the user specified an exact
      * range with start-end, we can't consider the right of the range as
@@ -124,85 +128,27 @@ rocksdb::Status BitmapString::BitPos(const std::string &raw_value, bool bit, int
   return rocksdb::Status::OK();
 }
 
-/* Count number of bits set in the binary array pointed by 's' and long
- * 'count' bytes. The implementation of this function is required to
- * work with a input string length up to 512 MB.
- * */
-size_t BitmapString::RawPopcount(const uint8_t *p, int64_t count) {
-  size_t bits = 0;
-
-  for (; count >= 8; p += 8, count -= 8) {
-    bits += __builtin_popcountll(*reinterpret_cast<const uint64_t *>(p));
-  }
-
-  if (count > 0) {
-    uint64_t v = 0;
-    __builtin_memcpy(&v, p, count);
-    bits += __builtin_popcountll(v);
-  }
-
-  return bits;
+std::pair<int64_t, int64_t> BitmapString::NormalizeRange(int64_t origin_start, int64_t origin_end, int64_t length) {
+  if (origin_start < 0) origin_start = length + origin_start;
+  if (origin_end < 0) origin_end = length + origin_end;
+  if (origin_start < 0) origin_start = 0;
+  if (origin_end < 0) origin_end = 0;
+  if (origin_end >= length) origin_end = length - 1;
+  return {origin_start, origin_end};
 }
 
-template <typename T = void>
-inline int ClzllWithEndian(uint64_t x) {
-  if constexpr (IsLittleEndian()) {
-    return __builtin_clzll(__builtin_bswap64(x));
-  } else if constexpr (IsBigEndian()) {
-    return __builtin_clzll(x);
-  } else {
-    static_assert(AlwaysFalse<T>);
+std::pair<int64_t, int64_t> BitmapString::NormalizeToByteRangeWithPaddingMask(bool is_bit, int64_t origin_start,
+                                                                              int64_t origin_end,
+                                                                              uint8_t *first_byte_neg_mask,
+                                                                              uint8_t *last_byte_neg_mask) {
+  DCHECK(origin_start <= origin_end);
+  if (is_bit) {
+    *first_byte_neg_mask = ~((1 << (8 - (origin_start & 7))) - 1) & 0xFF;
+    *last_byte_neg_mask = (1 << (7 - (origin_end & 7))) - 1;
+    origin_start >>= 3;
+    origin_end >>= 3;
   }
-}
-
-/* Return the position of the first bit set to one (if 'bit' is 1) or
- * zero (if 'bit' is 0) in the bitmap starting at 's' and long 'count' bytes.
- *
- * The function is guaranteed to return a value >= 0 if 'bit' is 0 since if
- * no zero bit is found, it returns count*8 assuming the string is zero
- * padded on the right. However if 'bit' is 1 it is possible that there is
- * not a single set bit in the bitmap. In this special case -1 is returned.
- * */
-int64_t BitmapString::RawBitpos(const uint8_t *c, int64_t count, bool bit) {
-  int64_t res = 0;
-
-  if (bit) {
-    int64_t ct = count;
-
-    for (; count >= 8; c += 8, count -= 8) {
-      uint64_t x = *reinterpret_cast<const uint64_t *>(c);
-      if (x != 0) {
-        return res + ClzllWithEndian(x);
-      }
-      res += 64;
-    }
-
-    if (count > 0) {
-      uint64_t v = 0;
-      __builtin_memcpy(&v, c, count);
-      res += v == 0 ? count * 8 : ClzllWithEndian(v);
-    }
-
-    if (res == ct * 8) {
-      return -1;
-    }
-  } else {
-    for (; count >= 8; c += 8, count -= 8) {
-      uint64_t x = *reinterpret_cast<const uint64_t *>(c);
-      if (x != (uint64_t)-1) {
-        return res + ClzllWithEndian(~x);
-      }
-      res += 64;
-    }
-
-    if (count > 0) {
-      uint64_t v = -1;
-      __builtin_memcpy(&v, c, count);
-      res += v == (uint64_t)-1 ? count * 8 : ClzllWithEndian(~v);
-    }
-  }
-
-  return res;
+  return {origin_start, origin_end};
 }
 
 rocksdb::Status BitmapString::Bitfield(const Slice &ns_key, std::string *raw_value,
@@ -222,7 +168,10 @@ rocksdb::Status BitmapString::Bitfield(const Slice &ns_key, std::string *raw_val
 
     ArrayBitfieldBitmap bitfield(first_byte);
     auto str = reinterpret_cast<const uint8_t *>(string_value.data() + first_byte);
-    auto s = bitfield.Set(first_byte, last_byte - first_byte, str);
+    auto s = bitfield.Set(/*byte_offset=*/first_byte, /*bytes=*/last_byte - first_byte, /*src=*/str);
+    if (!s.IsOK()) {
+      return rocksdb::Status::IOError(s.Msg());
+    }
 
     uint64_t unsigned_old_value = 0;
     if (op.encoding.IsSigned()) {
@@ -232,13 +181,19 @@ rocksdb::Status BitmapString::Bitfield(const Slice &ns_key, std::string *raw_val
     }
 
     uint64_t unsigned_new_value = 0;
-    auto &ret = rets->emplace_back();
-    if (BitfieldOp(op, unsigned_old_value, &unsigned_new_value).GetValue()) {
+    std::optional<BitfieldValue> &ret = rets->emplace_back();
+    StatusOr<bool> bitfield_op = BitfieldOp(op, unsigned_old_value, &unsigned_new_value);
+    if (!bitfield_op.IsOK()) {
+      return rocksdb::Status::InvalidArgument(bitfield_op.Msg());
+    }
+    if (bitfield_op.GetValue()) {
       if (op.type != BitfieldOperation::Type::kGet) {
         // never failed.
         s = bitfield.SetBitfield(op.offset, op.encoding.Bits(), unsigned_new_value);
+        CHECK(s.IsOK());
         auto dst = reinterpret_cast<uint8_t *>(string_value.data()) + first_byte;
         s = bitfield.Get(first_byte, last_byte - first_byte, dst);
+        CHECK(s.IsOK());
       }
 
       if (op.type == BitfieldOperation::Type::kSet) {
@@ -276,6 +231,9 @@ rocksdb::Status BitmapString::BitfieldReadOnly(const Slice &ns_key, const std::s
     ArrayBitfieldBitmap bitfield(first_byte);
     auto s = bitfield.Set(first_byte, last_byte - first_byte,
                           reinterpret_cast<const uint8_t *>(string_value.data() + first_byte));
+    if (!s.IsOK()) {
+      return rocksdb::Status::IOError(s.Msg());
+    }
 
     if (op.encoding.IsSigned()) {
       int64_t value = bitfield.GetSignedBitfield(op.offset, op.encoding.Bits()).GetValue();
