@@ -54,21 +54,15 @@ rocksdb::Status Json::write(Slice ns_key, JsonMetadata *metadata, const JsonValu
   return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
-rocksdb::Status Json::read(const Slice &ns_key, JsonMetadata *metadata, JsonValue *value) {
-  std::string bytes;
-  Slice rest;
-
-  auto s = GetMetadata(kRedisJson, ns_key, &bytes, metadata, &rest);
-  if (!s.ok()) return s;
-
-  if (metadata->format == JsonStorageFormat::JSON) {
-    auto origin_res = JsonValue::FromString(rest.ToStringView());
-    if (!origin_res) return rocksdb::Status::Corruption(origin_res.Msg());
-    *value = *std::move(origin_res);
-  } else if (metadata->format == JsonStorageFormat::CBOR) {
-    auto origin_res = JsonValue::FromCBOR(rest.ToStringView());
-    if (!origin_res) return rocksdb::Status::Corruption(origin_res.Msg());
-    *value = *std::move(origin_res);
+rocksdb::Status Json::parse(const JsonMetadata &metadata, const Slice &json_bytes, JsonValue *value) {
+  if (metadata.format == JsonStorageFormat::JSON) {
+    auto res = JsonValue::FromString(json_bytes.ToStringView());
+    if (!res) return rocksdb::Status::Corruption(res.Msg());
+    *value = *std::move(res);
+  } else if (metadata.format == JsonStorageFormat::CBOR) {
+    auto res = JsonValue::FromCBOR(json_bytes.ToStringView());
+    if (!res) return rocksdb::Status::Corruption(res.Msg());
+    *value = *std::move(res);
   } else {
     return rocksdb::Status::NotSupported("JSON storage format not supported");
   }
@@ -76,6 +70,16 @@ rocksdb::Status Json::read(const Slice &ns_key, JsonMetadata *metadata, JsonValu
   return rocksdb::Status::OK();
 }
 
+rocksdb::Status Json::read(const Slice &ns_key, JsonMetadata *metadata, JsonValue *value) {
+  std::string bytes;
+  Slice rest;
+
+  auto s = GetMetadata({kRedisJson}, ns_key, &bytes, metadata, &rest);
+  if (!s.ok()) return s;
+
+  return parse(*metadata, rest, value);
+}
+
 rocksdb::Status Json::create(const std::string &ns_key, JsonMetadata &metadata, const std::string &value) {
   auto json_res = JsonValue::FromString(value, storage_->GetConfig()->json_max_nesting_depth);
   if (!json_res) return rocksdb::Status::InvalidArgument(json_res.Msg());
@@ -101,7 +105,7 @@ rocksdb::Status Json::Info(const std::string &user_key, JsonStorageFormat *stora
   Slice rest;
   JsonMetadata metadata;
 
-  auto s = GetMetadata(kRedisJson, ns_key, &bytes, &metadata, &rest);
+  auto s = GetMetadata({kRedisJson}, ns_key, &bytes, &metadata, &rest);
   if (!s.ok()) return s;
 
   *storage_format = metadata.format;
@@ -512,4 +516,58 @@ rocksdb::Status Json::ObjLen(const std::string &user_key, const std::string &pat
   return rocksdb::Status::OK();
 }
 
+std::vector<rocksdb::Status> Json::MGet(const std::vector<std::string> &user_keys, const std::string &path,
+                                        std::vector<JsonValue> &results) {
+  std::vector<Slice> ns_keys;
+  std::vector<std::string> ns_keys_string;
+  ns_keys.resize(user_keys.size());
+  ns_keys_string.resize(user_keys.size());
+
+  for (size_t i = 0; i < user_keys.size(); i++) {
+    ns_keys_string[i] = AppendNamespacePrefix(user_keys[i]);
+    ns_keys[i] = Slice(ns_keys_string[i]);
+  }
+
+  std::vector<JsonValue> json_vals;
+  json_vals.resize(ns_keys.size());
+  auto statuses = readMulti(ns_keys, json_vals);
+
+  results.resize(ns_keys.size());
+  for (size_t i = 0; i < ns_keys.size(); i++) {
+    if (!statuses[i].ok()) {
+      continue;
+    }
+    auto res = json_vals[i].Get(path);
+
+    if (!res) {
+      statuses[i] = rocksdb::Status::Corruption(res.Msg());
+    } else {
+      results[i] = *std::move(res);
+    }
+  }
+  return statuses;
+}
+
+std::vector<rocksdb::Status> Json::readMulti(const std::vector<Slice> &ns_keys, std::vector<JsonValue> &values) {
+  rocksdb::ReadOptions read_options = storage_->DefaultMultiGetOptions();
+  LatestSnapShot ss(storage_);
+  read_options.snapshot = ss.GetSnapShot();
+
+  std::vector<rocksdb::Status> statuses(ns_keys.size());
+  std::vector<rocksdb::PinnableSlice> pin_values(ns_keys.size());
+  storage_->MultiGet(read_options, metadata_cf_handle_, ns_keys.size(), ns_keys.data(), pin_values.data(),
+                     statuses.data());
+  for (size_t i = 0; i < ns_keys.size(); i++) {
+    if (!statuses[i].ok()) continue;
+    Slice rest(pin_values[i].data(), pin_values[i].size());
+    JsonMetadata metadata;
+    statuses[i] = ParseMetadata({kRedisJson}, &rest, &metadata);
+    if (!statuses[i].ok()) continue;
+
+    statuses[i] = parse(metadata, rest, &values[i]);
+    if (!statuses[i].ok()) continue;
+  }
+  return statuses;
+}
+
 }  // namespace redis
