@@ -23,7 +23,6 @@
 #include <cmath>
 #include <cstddef>
 #include <cstdint>
-#include <limits>
 #include <optional>
 #include <string>
 
@@ -48,20 +47,11 @@ std::vector<rocksdb::Status> String::getRawValues(const std::vector<Slice> &keys
     if (!statuses[i].ok()) continue;
     (*raw_values)[i].assign(pin_values[i].data(), pin_values[i].size());
     Metadata metadata(kRedisNone, false);
-    auto s = metadata.Decode((*raw_values)[i]);
+    Slice slice = (*raw_values)[i];
+    auto s = ParseMetadata({kRedisString}, &slice, &metadata);
     if (!s.ok()) {
-      (*raw_values)[i].clear();
       statuses[i] = s;
-      continue;
-    }
-    if (metadata.Expired()) {
       (*raw_values)[i].clear();
-      statuses[i] = rocksdb::Status::NotFound(kErrMsgKeyExpired);
-      continue;
-    }
-    if (metadata.Type() != kRedisString && metadata.size > 0) {
-      (*raw_values)[i].clear();
-      statuses[i] = rocksdb::Status::InvalidArgument(kErrMsgWrongType);
       continue;
     }
   }
@@ -71,36 +61,37 @@ std::vector<rocksdb::Status> String::getRawValues(const std::vector<Slice> &keys
 rocksdb::Status String::getRawValue(const std::string &ns_key, std::string *raw_value) {
   raw_value->clear();
 
-  rocksdb::ReadOptions read_options;
-  LatestSnapShot ss(storage_);
-  read_options.snapshot = ss.GetSnapShot();
-  rocksdb::Status s = storage_->Get(read_options, metadata_cf_handle_, ns_key, raw_value);
+  auto s = GetRawMetadata(ns_key, raw_value);
   if (!s.ok()) return s;
 
   Metadata metadata(kRedisNone, false);
-  s = metadata.Decode(*raw_value);
-  if (!s.ok()) return s;
-  if (metadata.Expired()) {
-    raw_value->clear();
-    return rocksdb::Status::NotFound(kErrMsgKeyExpired);
-  }
-  if (metadata.Type() != kRedisString && metadata.size > 0) {
-    return rocksdb::Status::InvalidArgument(kErrMsgWrongType);
-  }
-  return rocksdb::Status::OK();
+  Slice slice = *raw_value;
+  return ParseMetadata({kRedisString}, &slice, &metadata);
 }
 
-rocksdb::Status String::getValue(const std::string &ns_key, std::string *value) {
+rocksdb::Status String::getValueAndExpire(const std::string &ns_key, std::string *value, uint64_t *expire) {
   value->clear();
 
   std::string raw_value;
   auto s = getRawValue(ns_key, &raw_value);
   if (!s.ok()) return s;
+
   size_t offset = Metadata::GetOffsetAfterExpire(raw_value[0]);
   *value = raw_value.substr(offset);
+
+  if (expire) {
+    Metadata metadata(kRedisString, false);
+    s = metadata.Decode(raw_value);
+    if (!s.ok()) return s;
+    *expire = metadata.expire;
+  }
   return rocksdb::Status::OK();
 }
 
+rocksdb::Status String::getValue(const std::string &ns_key, std::string *value) {
+  return getValueAndExpire(ns_key, value, nullptr);
+}
+
 std::vector<rocksdb::Status> String::getValues(const std::vector<Slice> &ns_keys, std::vector<std::string> *values) {
   auto statuses = getRawValues(ns_keys, values);
   for (size_t i = 0; i < ns_keys.size(); i++) {
@@ -187,20 +178,10 @@ rocksdb::Status String::GetEx(const std::string &user_key, std::string *value, u
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status String::GetSet(const std::string &user_key, const std::string &new_value, std::string *old_value) {
-  std::string ns_key = AppendNamespacePrefix(user_key);
-
-  LockGuard guard(storage_->GetLockManager(), ns_key);
-  rocksdb::Status s = getValue(ns_key, old_value);
-  if (!s.ok() && !s.IsNotFound()) return s;
-
-  std::string raw_value;
-  Metadata metadata(kRedisString, false);
-  metadata.Encode(&raw_value);
-  raw_value.append(new_value);
-  auto write_status = updateRawValue(ns_key, raw_value);
-  // prev status was used to tell whether old value was empty or not
-  return !write_status.ok() ? write_status : s;
+rocksdb::Status String::GetSet(const std::string &user_key, const std::string &new_value,
+                               std::optional<std::string> &old_value) {
+  auto s = Set(user_key, new_value, {/*ttl=*/0, StringSetType::NONE, /*get=*/true, /*keep_ttl=*/false}, old_value);
+  return s;
 }
 rocksdb::Status String::GetDel(const std::string &user_key, std::string *value) {
   std::string ns_key = AppendNamespacePrefix(user_key);
@@ -217,38 +198,85 @@ rocksdb::Status String::Set(const std::string &user_key, const std::string &valu
   return MSet(pairs, /*ttl=*/0, /*lock=*/true);
 }
 
+rocksdb::Status String::Set(const std::string &user_key, const std::string &value, StringSetArgs args,
+                            std::optional<std::string> &ret) {
+  uint64_t expire = 0;
+  std::string ns_key = AppendNamespacePrefix(user_key);
+
+  LockGuard guard(storage_->GetLockManager(), ns_key);
+  bool need_old_value = args.type != StringSetType::NONE || args.get || args.keep_ttl;
+  if (need_old_value) {
+    std::string old_value;
+    uint64_t old_expire = 0;
+    auto s = getValueAndExpire(ns_key, &old_value, &old_expire);
+    if (!s.ok() && !s.IsNotFound() && !s.IsInvalidArgument()) return s;
+    // GET option
+    if (args.get) {
+      if (s.IsInvalidArgument()) {
+        return s;
+      }
+      if (s.IsNotFound()) {
+        // if GET option given, the key didn't exist before: return nil
+        ret = std::nullopt;
+      } else {
+        // if GET option given: return The previous value of the key.
+        ret = std::make_optional(old_value);
+      }
+    }
+    // NX/XX option
+    if (args.type == StringSetType::NX && s.ok()) {
+      // if NX option given, the key already exist: return nil
+      if (!args.get) ret = std::nullopt;
+      return rocksdb::Status::OK();
+    } else if (args.type == StringSetType::XX && s.IsNotFound()) {
+      // if XX option given, the key didn't exist before: return nil
+      if (!args.get) ret = std::nullopt;
+      return rocksdb::Status::OK();
+    } else {
+      // if GET option not given, make ret not nil
+      if (!args.get) ret = "";
+    }
+    if (s.ok() && args.keep_ttl) {
+      // if KEEPTTL option given, use the old ttl
+      expire = old_expire;
+    }
+  } else {
+    // if no option given, make ret not nil
+    if (!args.get) ret = "";
+  }
+
+  // Handle expire time
+  if (args.ttl > 0) {
+    uint64_t now = util::GetTimeStampMS();
+    expire = now + args.ttl;
+  }
+
+  // Create new value
+  std::string new_raw_value;
+  Metadata metadata(kRedisString, false);
+  metadata.expire = expire;
+  metadata.Encode(&new_raw_value);
+  new_raw_value.append(value);
+  return updateRawValue(ns_key, new_raw_value);
+}
+
 rocksdb::Status String::SetEX(const std::string &user_key, const std::string &value, uint64_t ttl) {
-  std::vector<StringPair> pairs{StringPair{user_key, value}};
-  return MSet(pairs, /*ttl=*/ttl, /*lock=*/true);
+  std::optional<std::string> ret;
+  return Set(user_key, value, {ttl, StringSetType::NONE, /*get=*/false, /*keep_ttl=*/false}, ret);
 }
 
 rocksdb::Status String::SetNX(const std::string &user_key, const std::string &value, uint64_t ttl, bool *flag) {
-  std::vector<StringPair> pairs{StringPair{user_key, value}};
-  return MSetNX(pairs, ttl, flag);
+  std::optional<std::string> ret;
+  auto s = Set(user_key, value, {ttl, StringSetType::NX, /*get=*/false, /*keep_ttl=*/false}, ret);
+  *flag = ret.has_value();
+  return s;
 }
 
 rocksdb::Status String::SetXX(const std::string &user_key, const std::string &value, uint64_t ttl, bool *flag) {
-  *flag = false;
-  int exists = 0;
-  uint64_t expire = 0;
-  if (ttl > 0) {
-    uint64_t now = util::GetTimeStampMS();
-    expire = now + ttl;
-  }
-
-  std::string ns_key = AppendNamespacePrefix(user_key);
-  LockGuard guard(storage_->GetLockManager(), ns_key);
-  auto s = Exists({user_key}, &exists);
-  if (!s.ok()) return s;
-  if (exists != 1) return rocksdb::Status::OK();
-
-  *flag = true;
-  std::string raw_value;
-  Metadata metadata(kRedisString, false);
-  metadata.expire = expire;
-  metadata.Encode(&raw_value);
-  raw_value.append(value);
-  return updateRawValue(ns_key, raw_value);
+  std::optional<std::string> ret;
+  auto s = Set(user_key, value, {ttl, StringSetType::XX, /*get=*/false, /*keep_ttl=*/false}, ret);
+  *flag = ret.has_value();
+  return s;
 }
 
 rocksdb::Status String::SetRange(const std::string &user_key, size_t offset, const std::string &value,
