@@ -28,11 +28,12 @@
 #include <set>
 
 #include "db_util.h"
+#include "sample_helper.h"
 
 namespace redis {
 
 rocksdb::Status ZSet::GetMetadata(const Slice &ns_key, ZSetMetadata *metadata) {
-  return Database::GetMetadata(kRedisZSet, ns_key, metadata);
+  return Database::GetMetadata({kRedisZSet}, ns_key, metadata);
 }
 
 rocksdb::Status ZSet::Add(const Slice &user_key, ZAddFlags flags, MemberScores *mscores, uint64_t *added_cnt) {
@@ -178,7 +179,7 @@ rocksdb::Status ZSet::Pop(const Slice &user_key, int count, bool min, MemberScor
 
   auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
   iter->Seek(start_key);
-  // see comment in rangebyscore()
+  // see comment in RangeByScore()
   if (!min && (!iter->Valid() || !iter->key().starts_with(prefix_key))) {
     iter->SeekForPrev(start_key);
   }
@@ -248,7 +249,7 @@ rocksdb::Status ZSet::RangeByRank(const Slice &user_key, const RangeRankSpec &sp
   auto batch = storage_->GetWriteBatchBase();
   auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
   iter->Seek(start_key);
-  // see comment in rangebyscore()
+  // see comment in RangeByScore()
   if (spec.reversed && (!iter->Valid() || !iter->key().starts_with(prefix_key))) {
     iter->SeekForPrev(start_key);
   }
@@ -582,7 +583,7 @@ rocksdb::Status ZSet::Rank(const Slice &user_key, const Slice &member, bool reve
 
   auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
   iter->Seek(start_key);
-  // see comment in rangebyscore()
+  // see comment in RangeByScore()
   if (reversed && (!iter->Valid() || !iter->key().starts_with(prefix_key))) {
     iter->SeekForPrev(start_key);
   }
@@ -699,6 +700,52 @@ rocksdb::Status ZSet::Inter(const std::vector<KeyWeight> &keys_weights, Aggregat
   return rocksdb::Status::OK();
 }
 
+rocksdb::Status ZSet::InterCard(const std::vector<std::string> &user_keys, uint64_t limit, uint64_t *inter_cnt) {
+  std::vector<std::string> lock_keys;
+  lock_keys.reserve(user_keys.size());
+  for (const auto &user_key : user_keys) {
+    std::string ns_key = AppendNamespacePrefix(user_key);
+    lock_keys.emplace_back(std::move(ns_key));
+  }
+  MultiLockGuard guard(storage_->GetLockManager(), lock_keys);
+
+  std::vector<MemberScores> mscores_list;
+  mscores_list.reserve(user_keys.size());
+  RangeScoreSpec spec;
+  for (const auto &user_key : user_keys) {
+    MemberScores mscores;
+    auto s = RangeByScore(user_key, spec, &mscores, nullptr);
+    if (!s.ok() || mscores.empty()) return s;
+    mscores_list.emplace_back(std::move(mscores));
+  }
+  std::sort(mscores_list.begin(), mscores_list.end(),
+            [](const MemberScores &v1, const MemberScores &v2) { return v1.size() < v2.size(); });
+
+  auto base_mscores = mscores_list[0];
+  std::map<std::string, size_t> member_counters;
+  uint64_t cardinality = 0;
+  for (const auto &base_ms : base_mscores) {
+    member_counters[base_ms.member] = 1;
+    for (size_t i = 1; i < mscores_list.size(); i++) {
+      for (const auto &ms : mscores_list[i]) {
+        if (base_ms.member == ms.member) {
+          member_counters[ms.member]++;
+          break;
+        }
+      }
+    }
+    if (member_counters[base_ms.member] == mscores_list.size()) {
+      cardinality++;
+      if (limit > 0 && cardinality >= limit) {
+        *inter_cnt = limit;
+        return rocksdb::Status::OK();
+      };
+    }
+  }
+  *inter_cnt = cardinality;
+  return rocksdb::Status::OK();
+}
+
 rocksdb::Status ZSet::UnionStore(const Slice &dst, const std::vector<KeyWeight> &keys_weights,
                                  AggregateMethod aggregate_method, uint64_t *saved_cnt) {
   *saved_cnt = 0;
@@ -805,4 +852,99 @@ rocksdb::Status ZSet::MGet(const Slice &user_key, const std::vector<Slice> &memb
   return rocksdb::Status::OK();
 }
 
+rocksdb::Status ZSet::GetAllMemberScores(const Slice &user_key, std::vector<MemberScore> *member_scores) {
+  member_scores->clear();
+  std::string ns_key = AppendNamespacePrefix(user_key);
+  ZSetMetadata metadata(false);
+  rocksdb::Status s = GetMetadata(ns_key, &metadata);
+  if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
+
+  std::string prefix_key = InternalKey(ns_key, "", metadata.version, storage_->IsSlotIdEncoded()).Encode();
+  std::string next_version_prefix_key =
+      InternalKey(ns_key, "", metadata.version + 1, storage_->IsSlotIdEncoded()).Encode();
+
+  rocksdb::ReadOptions read_options = storage_->DefaultScanOptions();
+  LatestSnapShot ss(storage_);
+  read_options.snapshot = ss.GetSnapShot();
+
+  rocksdb::Slice upper_bound(next_version_prefix_key);
+  rocksdb::Slice lower_bound(prefix_key);
+  read_options.iterate_upper_bound = &upper_bound;
+  read_options.iterate_lower_bound = &lower_bound;
+
+  auto iter = util::UniqueIterator(storage_, read_options, score_cf_handle_);
+
+  for (iter->Seek(prefix_key); iter->Valid() && iter->key().starts_with(prefix_key); iter->Next()) {
+    InternalKey ikey(iter->key(), storage_->IsSlotIdEncoded());
+    Slice score_key = ikey.GetSubKey();
+    double score = NAN;
+    GetDouble(&score_key, &score);
+    member_scores->emplace_back(MemberScore{score_key.ToString(), score});
+  }
+
+  return rocksdb::Status::OK();
+}
+
+rocksdb::Status ZSet::RandMember(const Slice &user_key, int64_t command_count,
+                                 std::vector<MemberScore> *member_scores) {
+  if (command_count == 0) {
+    return rocksdb::Status::OK();
+  }
+
+  uint64_t count = command_count > 0 ? static_cast<uint64_t>(command_count) : static_cast<uint64_t>(-command_count);
+  bool unique = (command_count >= 0);
+
+  std::string ns_key = AppendNamespacePrefix(user_key);
+  ZSetMetadata metadata(false);
+  rocksdb::Status s = GetMetadata(ns_key, &metadata);
+  if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
+  if (metadata.size == 0) return rocksdb::Status::OK();
+
+  return ExtractRandMemberFromSet<MemberScore>(
+      unique, count,
+      [this, user_key](std::vector<MemberScore> *scores) -> rocksdb::Status {
+        return this->GetAllMemberScores(user_key, scores);
+      },
+      member_scores);
+}
+
+rocksdb::Status ZSet::Diff(const std::vector<Slice> &keys, MemberScores *members) {
+  members->clear();
+  MemberScores source_member_scores;
+  RangeScoreSpec spec;
+  uint64_t first_element_size = 0;
+  auto s = RangeByScore(keys[0], spec, &source_member_scores, &first_element_size);
+  if (!s.ok()) return s;
+
+  if (first_element_size == 0) {
+    return rocksdb::Status::OK();
+  }
+
+  std::set<std::string> exclude_members;
+  MemberScores target_member_scores;
+  for (size_t i = 1; i < keys.size(); i++) {
+    uint64_t size = 0;
+    s = RangeByScore(keys[i], spec, &target_member_scores, &size);
+    if (!s.ok()) return s;
+    for (auto &member_score : target_member_scores) {
+      exclude_members.emplace(std::move(member_score.member));
+    }
+    target_member_scores.clear();
+  }
+  for (const auto &member_score : source_member_scores) {
+    if (exclude_members.find(member_score.member) == exclude_members.end()) {
+      members->push_back(member_score);
+    }
+  }
+  return rocksdb::Status::OK();
+}
+
+rocksdb::Status ZSet::DiffStore(const Slice &dst, const std::vector<Slice> &keys, uint64_t *stored_count) {
+  MemberScores mscores;
+  auto s = Diff(keys, &mscores);
+  if (!s.ok()) return s;
+  *stored_count = mscores.size();
+  return Overwrite(dst, mscores);
+}
+
 }  // namespace redis
