@@ -20,7 +20,9 @@
 
 #include <gtest/gtest.h>
 
+#include <chrono>
 #include <memory>
+#include <thread>
 
 #include "storage/redis_metadata.h"
 #include "test_base.h"
@@ -67,8 +69,8 @@ TEST(Metadata, EncodeAndDecode) {
 class RedisTypeTest : public TestBase {
  public:
   RedisTypeTest() {
-    redis_ = std::make_unique<redis::Database>(storage_, "default_ns");
-    hash_ = std::make_unique<redis::Hash>(storage_, "default_ns");
+    redis_ = std::make_unique<redis::Database>(storage_.get(), "default_ns");
+    hash_ = std::make_unique<redis::Hash>(storage_.get(), "default_ns");
     key_ = "test-redis-type";
     fields_ = {"test-hash-key-1", "test-hash-key-2", "test-hash-key-3"};
     values_ = {"hash-test-value-1", "hash-test-value-2", "hash-test-value-3"};
@@ -90,7 +92,7 @@ TEST_F(RedisTypeTest, GetMetadata) {
   EXPECT_TRUE(s.ok() && fvs.size() == ret);
   HashMetadata metadata;
   std::string ns_key = redis_->AppendNamespacePrefix(key_);
-  s = redis_->GetMetadata(kRedisHash, ns_key, &metadata);
+  s = redis_->GetMetadata({kRedisHash}, ns_key, &metadata);
   EXPECT_EQ(fvs.size(), metadata.size);
   s = redis_->Del(key_);
   EXPECT_TRUE(s.ok());
@@ -114,6 +116,81 @@ TEST_F(RedisTypeTest, Expire) {
   s = redis_->Del(key_);
 }
 
+TEST_F(RedisTypeTest, ExpireTime) {
+  uint64_t ret = 0;
+  std::vector<FieldValue> fvs;
+  for (size_t i = 0; i < fields_.size(); i++) {
+    fvs.emplace_back(fields_[i].ToString(), values_[i].ToString());
+  }
+  rocksdb::Status s = hash_->MSet(key_, fvs, false, &ret);
+  EXPECT_TRUE(s.ok() && fvs.size() == ret);
+  int64_t now = 0;
+  rocksdb::Env::Default()->GetCurrentTime(&now);
+  uint64_t ms_offset = 2314;
+  uint64_t expire_timestamp_ms = now * 1000 + ms_offset;
+  s = redis_->Expire(key_, expire_timestamp_ms);
+  EXPECT_TRUE(s.ok());
+  uint64_t timestamp = 0;
+  s = redis_->GetExpireTime(key_, &timestamp);
+  EXPECT_TRUE(s.ok() && timestamp != 0);
+  if (METADATA_ENCODING_VERSION != 0) {
+    EXPECT_EQ(timestamp, expire_timestamp_ms);
+  } else {
+    EXPECT_EQ(timestamp, Metadata::ExpireMsToS(expire_timestamp_ms) * 1000);
+  }
+  s = redis_->Del(key_);
+}
+
+TEST_F(RedisTypeTest, ExpireTimeKeyNoExpireTime) {
+  uint64_t ret = 0;
+  std::vector<FieldValue> fvs;
+  for (size_t i = 0; i < fields_.size(); i++) {
+    fvs.emplace_back(fields_[i].ToString(), values_[i].ToString());
+  }
+  rocksdb::Status s = hash_->MSet(key_, fvs, false, &ret);
+  EXPECT_TRUE(s.ok() && fvs.size() == ret);
+  uint64_t timestamp = 0;
+  s = redis_->GetExpireTime(key_, &timestamp);
+  EXPECT_TRUE(s.ok() && timestamp == 0);
+  s = redis_->Del(key_);
+}
+
+TEST_F(RedisTypeTest, ExpireTimeKeyExpired) {
+  uint64_t ret = 0;
+  std::vector<FieldValue> fvs;
+  for (size_t i = 0; i < fields_.size(); i++) {
+    fvs.emplace_back(fields_[i].ToString(), values_[i].ToString());
+  }
+  rocksdb::Status s = hash_->MSet(key_, fvs, false, &ret);
+  EXPECT_TRUE(s.ok() && fvs.size() == ret);
+  int64_t now = 0;
+  rocksdb::Env::Default()->GetCurrentTime(&now);
+  uint64_t ms_offset = 1120;
+  uint64_t expire_timestamp_ms = now * 1000 + ms_offset;
+  s = redis_->Expire(key_, expire_timestamp_ms);
+  EXPECT_TRUE(s.ok());
+  std::this_thread::sleep_for(std::chrono::milliseconds(2000));
+  uint64_t timestamp = 0;
+  s = redis_->GetExpireTime(key_, &timestamp);
+  EXPECT_TRUE(s.IsNotFound() && timestamp == 0);
+  s = redis_->Del(key_);
+}
+
+TEST_F(RedisTypeTest, ExpireTimeKeyNotExisted) {
+  uint64_t ret = 0;
+  std::vector<FieldValue> fvs;
+  for (size_t i = 0; i < fields_.size(); i++) {
+    fvs.emplace_back(fields_[i].ToString(), values_[i].ToString());
+  }
+  rocksdb::Status s = hash_->MSet(key_, fvs, false, &ret);
+  EXPECT_TRUE(s.ok() && fvs.size() == ret);
+  uint64_t timestamp = 0;
+  s = redis_->GetExpireTime(key_ + "test", &timestamp);
+  EXPECT_TRUE(s.IsNotFound() && timestamp == 0);
+
+  s = redis_->Del(key_);
+}
+
 TEST(Metadata, MetadataDecodingBackwardCompatibleSimpleKey) {
   auto expire_at = (util::GetTimeStamp() + 10) * 1000;
   Metadata md_old(kRedisString, true, false);
