@@ -27,7 +27,7 @@
 
 class RedisZSetTest : public TestBase {
  protected:
-  RedisZSetTest() { zset_ = std::make_unique<redis::ZSet>(storage_, "zset_ns"); }
+  RedisZSetTest() { zset_ = std::make_unique<redis::ZSet>(storage_.get(), "zset_ns"); }
   ~RedisZSetTest() override = default;
 
   void SetUp() override {
@@ -433,3 +433,183 @@ TEST_F(RedisZSetTest, Rank) {
   }
   auto s = zset_->Del(key_);
 }
+
+TEST_F(RedisZSetTest, RandMember) {
+  uint64_t ret = 0;
+  {
+    std::vector<MemberScore> in_mscores;
+    in_mscores.reserve(fields_.size());
+    for (size_t i = 0; i < fields_.size(); i++) {
+      in_mscores.emplace_back(MemberScore{fields_[i].ToString(), scores_[i]});
+    }
+    zset_->Add(key_, ZAddFlags::Default(), &in_mscores, &ret);
+    EXPECT_EQ(static_cast<int>(fields_.size()), ret);
+  }
+
+  std::unordered_map<std::string, double> member_map;
+  for (size_t i = 0; i < fields_.size(); i++) {
+    member_map[fields_[i].ToString()] = scores_[i];
+  }
+
+  // count = 0
+  {
+    std::vector<MemberScore> mscores;
+    rocksdb::Status s = zset_->RandMember(key_, 0, &mscores);
+    EXPECT_EQ(0, mscores.size());
+    EXPECT_TRUE(s.ok());
+  }
+
+  // count = 1/-1
+  for (int64_t count : {1, -1}) {
+    std::vector<MemberScore> mscores;
+    rocksdb::Status s = zset_->RandMember(key_, count, &mscores);
+    EXPECT_EQ(1, mscores.size());
+    EXPECT_TRUE(s.ok());
+    EXPECT_NE(member_map.find(mscores[0].member), member_map.end());
+  }
+
+  auto no_duplicate_members = [](const std::vector<MemberScore> &mscores) {
+    std::unordered_set<std::string> member_set;
+    for (const auto &mscore : mscores) {
+      if (member_set.find(mscore.member) != member_set.end()) {
+        return false;
+      }
+      member_set.insert(mscore.member);
+    }
+    return true;
+  };
+
+  auto no_non_exist_members = [&member_map](const std::vector<MemberScore> &mscores) {
+    for (const auto &mscore : mscores) {
+      const auto find_res = member_map.find(mscore.member);
+      if (find_res == member_map.end() || find_res->second != mscore.score) {
+        return false;
+      }
+    }
+    return true;
+  };
+
+  // count > 1, but count <= fields_.size()
+  for (int64_t count : {
+           static_cast<int64_t>(fields_.size()),
+           static_cast<int64_t>(fields_.size() / 2),
+       }) {
+    std::vector<MemberScore> mscores;
+    rocksdb::Status s = zset_->RandMember(key_, count, &mscores);
+    EXPECT_EQ(static_cast<size_t>(count), mscores.size());
+    EXPECT_TRUE(s.ok());
+    ASSERT_TRUE(no_non_exist_members(mscores));
+    ASSERT_TRUE(no_duplicate_members(mscores));
+  }
+
+  // count < -1, but count >= -fields_.size()
+  for (int64_t count : {
+           -static_cast<int64_t>(fields_.size()),
+           -static_cast<int64_t>(fields_.size() / 2),
+       }) {
+    std::vector<MemberScore> mscores;
+    rocksdb::Status s = zset_->RandMember(key_, count, &mscores);
+    EXPECT_EQ(static_cast<size_t>(-count), mscores.size());
+    EXPECT_TRUE(s.ok());
+    ASSERT_TRUE(no_non_exist_members(mscores));
+  }
+
+  // cout < -fields_.size() or count > fields_.size()
+
+  for (int64_t count : {
+           static_cast<int64_t>(fields_.size() + 10),
+           -static_cast<int64_t>(fields_.size() + 10),
+       }) {
+    std::vector<MemberScore> mscores;
+    rocksdb::Status s = zset_->RandMember(key_, count, &mscores);
+    EXPECT_TRUE(s.ok());
+    ASSERT_TRUE(no_non_exist_members(mscores));
+    if (count > 0) {
+      EXPECT_EQ(fields_.size(), mscores.size());
+      ASSERT_TRUE(no_duplicate_members(mscores));
+    } else {
+      EXPECT_EQ(static_cast<size_t>(-count), mscores.size());
+    }
+  }
+
+  auto s = zset_->Del(key_);
+  EXPECT_TRUE(s.ok());
+}
+
+TEST_F(RedisZSetTest, Diff) {
+  uint64_t ret = 0;
+
+  std::string k1 = "key1";
+  std::vector<MemberScore> k1_mscores = {{"a", -100.1}, {"b", -100.1}, {"c", 0}, {"d", 1.234}};
+
+  std::string k2 = "key2";
+  std::vector<MemberScore> k2_mscores = {{"c", -150.1}};
+
+  std::string k3 = "key3";
+  std::vector<MemberScore> k3_mscores = {{"a", -1000.1}, {"c", -100.1}, {"e", 8000.9}};
+
+  auto s = zset_->Add(k1, ZAddFlags::Default(), &k1_mscores, &ret);
+  EXPECT_EQ(ret, 4);
+  zset_->Add(k2, ZAddFlags::Default(), &k2_mscores, &ret);
+  EXPECT_EQ(ret, 1);
+  zset_->Add(k3, ZAddFlags::Default(), &k3_mscores, &ret);
+  EXPECT_EQ(ret, 3);
+
+  std::vector<MemberScore> mscores;
+  zset_->Diff({k1, k2, k3}, &mscores);
+
+  EXPECT_EQ(2, mscores.size());
+  std::vector<MemberScore> expected_mscores = {{"b", -100.1}, {"d", 1.234}};
+  int index = 0;
+  for (const auto &mscore : expected_mscores) {
+    EXPECT_EQ(mscore.member, mscores[index].member);
+    EXPECT_EQ(mscore.score, mscores[index].score);
+    index++;
+  }
+
+  s = zset_->Del(k1);
+  EXPECT_TRUE(s.ok());
+  s = zset_->Del(k2);
+  EXPECT_TRUE(s.ok());
+  s = zset_->Del(k3);
+  EXPECT_TRUE(s.ok());
+}
+
+TEST_F(RedisZSetTest, DiffStore) {
+  uint64_t ret = 0;
+
+  std::string k1 = "key1";
+  std::vector<MemberScore> k1_mscores = {{"a", -100.1}, {"b", -100.1}, {"c", 0}, {"d", 1.234}};
+
+  std::string k2 = "key2";
+  std::vector<MemberScore> k2_mscores = {{"c", -150.1}};
+
+  auto s = zset_->Add(k1, ZAddFlags::Default(), &k1_mscores, &ret);
+  EXPECT_EQ(ret, 4);
+  zset_->Add(k2, ZAddFlags::Default(), &k2_mscores, &ret);
+  EXPECT_EQ(ret, 1);
+
+  uint64_t stored_count = 0;
+  zset_->DiffStore("zsetdiff", {k1, k2}, &stored_count);
+  EXPECT_EQ(stored_count, 3);
+
+  RangeScoreSpec spec;
+  std::vector<MemberScore> mscores;
+  zset_->RangeByScore("zsetdiff", spec, &mscores, nullptr);
+  EXPECT_EQ(mscores.size(), 3);
+
+  std::vector<MemberScore> expected_mscores = {{"a", -100.1}, {"b", -100.1}, {"d", 1.234}};
+  int index = 0;
+  for (const auto &mscore : expected_mscores) {
+    EXPECT_EQ(mscore.member, mscores[index].member);
+    EXPECT_EQ(mscore.score, mscores[index].score);
+    index++;
+  }
+
+  s = zset_->Del(k1);
+  EXPECT_TRUE(s.ok());
+  s = zset_->Del(k2);
+  EXPECT_TRUE(s.ok());
+  s = zset_->Del("zsetdiff");
+  EXPECT_TRUE(s.ok());
+}
